@@ -1,13 +1,9 @@
 import urllib
 from .sd_card_device import SDCardDevice
-from ..helpers import get_xpub_fingerprint
+from ..util.xpub import get_xpub_fingerprint
+from ..helpers import to_ascii20
 
-
-CC_TYPES = {
-    'legacy': 'BIP45',
-    'p2sh-segwit': 'P2WSH-P2SH',
-    'bech32': 'P2WSH'
-}
+CC_TYPES = {"legacy": "BIP45", "p2sh-segwit": "P2WSH-P2SH", "bech32": "P2WSH"}
 
 
 class ColdCard(SDCardDevice):
@@ -15,7 +11,7 @@ class ColdCard(SDCardDevice):
     name = "ColdCard"
 
     sd_card_support = True
-    wallet_export_type = 'file'
+    wallet_export_type = "file"
     supports_hwi_multisig_display_address = True
 
     def __init__(self, name, alias, keys, fullpath, manager):
@@ -26,17 +22,13 @@ class ColdCard(SDCardDevice):
         return psbts
 
     def export_wallet(self, wallet):
-        CC_TYPES = {
-            'legacy': 'BIP45',
-            'p2sh-segwit': 'P2WSH-P2SH',
-            'bech32': 'P2WSH'
-        }
+        CC_TYPES = {"legacy": "BIP45", "p2sh-segwit": "P2WSH-P2SH", "bech32": "P2WSH"}
         # try to find at least one derivation
         # cc assume the same derivation for all keys :(
         derivation = None
         # find correct key
         for k in wallet.keys:
-            if k in self.keys and k.derivation != '':
+            if k in self.keys and k.derivation != "":
                 derivation = k.derivation.replace("h", "'")
                 break
         if derivation is None:
@@ -47,14 +39,17 @@ Name: {}
 Policy: {} of {}
 Derivation: {}
 Format: {}
-""".format(wallet.name, wallet.sigs_required,
-            len(wallet.keys), derivation,
-            CC_TYPES[wallet.address_type]
-           )
+""".format(
+            to_ascii20(wallet.name),
+            wallet.sigs_required,
+            len(wallet.keys),
+            derivation,
+            CC_TYPES[wallet.address_type],
+        )
         for k in wallet.keys:
             # cc assumes fingerprint is known
             fingerprint = k.fingerprint
-            if fingerprint == '':
+            if fingerprint == "":
                 fingerprint = get_xpub_fingerprint(k.xpub).hex()
             cc_file += "{}: {}\n".format(fingerprint.upper(), k.xpub)
         return urllib.parse.quote(cc_file)
